@@ -1,5 +1,9 @@
 from enum import Enum
 
+import pandas as pd
+
+from sarus_data_spec import typing as st
+
 DATA = 'data'
 USER_COLUMN = 'sarus_protected_entity'
 WEIGHTS = 'sarus_weights'
@@ -14,6 +18,7 @@ TO_MERGE = 'fks_for_merging'
 LIST_VALUES = 'sarus_list_values'
 ARRAY_VALUES = 'sarus_array_values'
 OPTIONAL_VALUE = 'sarus_optional_value'
+CONSTRAINED_VALUE = 'sarus_constrained_value'
 
 
 class StructKind(Enum):
@@ -33,9 +38,13 @@ FLOAT_DISTRIBUTION = 'distribution_model'
 FLOAT_DIST_PARAMS = 'parameters'
 MAX_MAX_MULT = 'max_max_multiplicity'
 
-# name of the dataset
+# constants for dataset properties
 DATASET_SLUGNAME = 'slugname'
 
+# constants for schema properties
+PRIMARY_KEYS = 'primary_keys'
+FOREIGN_KEYS = 'foreign_keys'
+
 # names when transforming datetime type in struct
 DATETIME_YEAR = 'year'
 DATETIME_MONTH = 'month'
@@ -45,8 +54,9 @@ DATETIME_MINUTES = 'minutes'
 DATETIME_SECONDS = 'seconds'
 
 
-# Statuses constants
+# sql, to_sql status
 TO_SQL_TASK = "sql_preparation"
+SQL_TASK = 'sql'
 
 # Big Data Status
 BIG_DATA_TASK = 'big_data_dataset'
@@ -61,6 +71,10 @@ TO_PARQUET_TASK = 'to_parquet'
 CACHE = TO_PARQUET_TASK
 CACHE_PATH = 'path'
 COMPUTATION_QUEUED = 'computation_queued'
+TO_SQL_CACHING_TASK = 'to_sql_caching'
+SQL_CACHING_URI = 'sql_caching_uri'
+TABLE_MAPPING = 'table_mapping'
+EXTENDED_TABLE_MAPPING = 'extended_table_mapping'
 
 # Caching infos for scalar
 CACHE_TYPE = 'cache_type'
@@ -87,9 +101,12 @@ QB_TASK = 'query_builder'
 LINKS_TASK = 'links_statistics'
 SYNTHETIC_TASK = 'synthetic'
 
+
 # Privacy
 PEP_TOKEN = "pep_token"
 PRIVACY_LIMIT = "privacy_limit"
+CONSTRAINT_KIND = "constraint_kind"
+BEST_ALTERNATIVE = "best_alternative"
 
 # QUERYBUILDER
 QUERIES = 'queries'
@@ -102,8 +119,28 @@ RELATIONSHIP_SPEC = "relationship_spec"
 
 
 # SYNTHETIC DATA
+SYNTHETIC_MODEL = 'sarus_synthetic_model'
+TRAIN_CORRELATIONS = 'train_correlations'
+
+
 class SyntheticDataSettings:
     """Namespace for SD generation settings"""
 
     BATCH_SIZE = 64
     EPOCHS = 10
+
+
+# map between sqlalchemy dialect names to sarus notion of dialects
+SQLALCHEMY_DIALECT_MAP = {
+    "postgresql": st.SQLDialect.POSTGRES,
+    "mssql": st.SQLDialect.SQL_SERVER,
+    "mysql": st.SQLDialect.MY_SQL,
+    "sqlite": st.SQLDialect.SQLLITE,
+    "oracle": st.SQLDialect.ORACLE,
+    "bigquery": st.SQLDialect.BIG_QUERY,
+    "redshift": st.SQLDialect.REDSHIFT,
+    "hive": st.SQLDialect.HIVE,
+}
+
+# List of types a Dataset can be converted to
+DATASET_TYPES = [pd.DataFrame]
