@@ -84,11 +84,7 @@ class Dataset(Referring[sp.Dataset]):
 
     def is_synthetic(self) -> bool:
         """Is the dataset synthetic."""
-        if self.is_transformed():
-            transform = self.transform()
-            return transform.protobuf().spec.HasField("synthetic")
-        else:
-            return False
+        return self.manager().dataspec_validator().is_synthetic(self)
 
     def is_protected(self) -> bool:
         """Is the dataset protected."""
@@ -100,11 +96,15 @@ class Dataset(Referring[sp.Dataset]):
 
     def pep_token(self) -> Optional[str]:
         """Returns the dataset PEP token."""
-        return self.manager().query_manager().pep_token(self)
+        return self.manager().dataspec_validator().pep_token(self)
+
+    def is_dp(self) -> bool:
+        """Is the dataspec the result of a DP transform"""
+        return self.manager().dataspec_validator().is_dp(self)
 
     def is_public(self) -> bool:
         """Is the dataset public."""
-        return self.manager().query_manager().is_public(self)
+        return self.manager().dataspec_validator().is_public(self)
 
     def is_remote(self) -> bool:
         """Is the dataspec a remotely defined dataset."""
@@ -216,12 +216,12 @@ class Dataset(Referring[sp.Dataset]):
     ) -> Optional[st.DataSpec]:
         return (
             self.manager()
-            .query_manager()
+            .dataspec_rewriter()
             .variant(self, kind, public_context, privacy_limit)
         )
 
     def variants(self) -> Collection[st.DataSpec]:
-        return self.manager().query_manager().variants(self)
+        return self.manager().dataspec_rewriter().variants(self)
 
     def private_queries(self) -> List[st.PrivateQuery]:
         """Return the list of PrivateQueries used in a Dataspec's transform.
@@ -234,7 +234,7 @@ class Dataset(Referring[sp.Dataset]):
         an account, they are only accounted once (ask @cgastaud for more on
         accounting).
         """
-        return self.manager().query_manager().private_queries(self)
+        return self.manager().dataspec_validator().private_queries(self)
 
     def spec(self) -> str:
         return cast(str, self.protobuf().spec.WhichOneof('spec'))
