@@ -1,4 +1,3 @@
-import asyncio
 import logging
 import typing as t
 
@@ -21,6 +20,12 @@ class TypedApiManager(Manager, t.Protocol):
     async def async_value_op(self, scalar: st.Scalar) -> st.DataSpecValue:
         ...
 
+    def computation_timeout(self, dataspec: st.DataSpec) -> int:
+        ...
+
+    def computation_max_delay(self, dataspec: st.DataSpec) -> int:
+        ...
+
 
 class ApiComputation(BaseComputation[T], DelegatedComputation):
     def __init__(self, manager: TypedApiManager):
@@ -42,18 +47,13 @@ class ApiComputation(BaseComputation[T], DelegatedComputation):
         already done, so the manager just waits for the task to
         be completed"""
 
-        for _ in range(100):
-            logger.debug(f'POLLING {self.task_name} {dataspec.uuid()}')
-            status = self.status(dataspec=dataspec)
-            if status is None:
-                await asyncio.sleep(1)
-                continue
-            else:
-                break
+        stage = await self.wait_for_computation(
+            dataspec=dataspec,
+            current_stage='pending',
+            timeout=self.manager().computation_timeout(dataspec),
+            max_delay=self.manager().computation_max_delay(dataspec),
+        )
 
-        assert status
-        stage = status.task(self.task_name)
-        assert stage
         if stage.pending():
             # TODO push error status to worker ?
             stt.error(
@@ -69,9 +69,11 @@ class ApiComputation(BaseComputation[T], DelegatedComputation):
                 },
             )
             raise stt.DataSpecErrorStatus(
-                relaunch=True,
-                error_msg="Pending time out for task"
-                f" {self.task_name} on dataspec {dataspec}",
+                (
+                    True,
+                    "Pending time out for task"
+                    f" {self.task_name} on dataspec {dataspec}",
+                )
             )
 
         return await self.complete_task(dataspec)
@@ -82,19 +84,12 @@ class ApiComputation(BaseComputation[T], DelegatedComputation):
         of the task. After a given timeout, an error is raised.
         """
 
-        for _ in range(100):
-            logger.debug(f'POLLING {self.task_name} {dataspec.uuid()}')
-            status = self.status(dataspec=dataspec)
-            assert status
-            stage = status.task(self.task_name)
-            assert stage
-            if stage.processing():
-                await asyncio.sleep(1)
-                continue
-            else:
-                break
-
-        assert stage
+        stage = await self.wait_for_computation(
+            dataspec=dataspec,
+            current_stage='processing',
+            timeout=self.manager().computation_timeout(dataspec),
+            max_delay=self.manager().computation_max_delay(dataspec),
+        )
         if stage.processing() or stage.pending():
             # TODO push error status to worker ?
             stt.error(
@@ -110,9 +105,11 @@ class ApiComputation(BaseComputation[T], DelegatedComputation):
                 },
             )
             raise stt.DataSpecErrorStatus(
-                relaunch=True,
-                error_msg="Processing time out for task"
-                f" {self.task_name} on dataspec {dataspec}",
+                (
+                    True,
+                    "Processing time out for task"
+                    f" {self.task_name} on dataspec {dataspec}",
+                )
             )
 
         # if the stage is an error, it is complete_task
