@@ -12,6 +12,9 @@ import pyarrow as pa
 from sarus_data_spec.arrow.array import convert_record_batch
 from sarus_data_spec.manager.asyncio.utils import sync, sync_iterator
 from sarus_data_spec.manager.base import Base
+from sarus_data_spec.manager.ops.asyncio.processor.external.external_op import (  # noqa: E501
+    async_compute_external_value,
+)
 from sarus_data_spec.manager.typing import Computation, Manager
 from sarus_data_spec.schema import Schema
 import sarus_data_spec.protobuf as sp
@@ -263,6 +266,47 @@ class BaseAsyncManager(Base):
         nester = nest(signature)
         return tf.data.TFRecordDataset(filenames).map(deserializer).map(nester)
 
+    def mock_value(
+        self,
+        transform: st.Transform,
+        *arguments: st.DataSpec,
+        **named_arguments: st.DataSpec,
+    ) -> t.Any:
+        """Compute the mock value of an external transform applied on
+        Dataspecs.
+        """
+        assert transform.is_external()
+        mock_args = [arg.variant(st.ConstraintKind.MOCK) for arg in arguments]
+        named_mock_args = {
+            name: arg.variant(st.ConstraintKind.MOCK)
+            for name, arg in named_arguments.items()
+        }
+
+        if any([ds is None for ds in mock_args]) or any(
+            [ds is None for ds in named_mock_args.values()]
+        ):
+            raise ValueError(
+                "Cannot infer the output type of the external "
+                "transform because one of the parents has a None MOCK."
+            )
+
+        typed_mock_args = [t.cast(st.DataSpec, ds) for ds in mock_args]
+        typed_named_mock_args = {
+            name: t.cast(st.DataSpec, ds)
+            for name, ds in named_mock_args.items()
+        }
+
+        try:
+            mock_value = sync(
+                async_compute_external_value(
+                    transform, *typed_mock_args, **typed_named_mock_args
+                )
+            )
+        except Exception as e:
+            raise e
+
+        return mock_value
+
 
 async def write_tf_batch(
     filename: str,
@@ -338,9 +382,7 @@ class BaseComputation(Computation[T]):
                     'relaunch': str(exception.relaunch),
                 },
             )
-            raise stt.DataSpecErrorStatus(
-                (exception.relaunch, traceback.format_exc())
-            )
+            raise
         except Exception:
             stt.error(
                 dataspec=dataspec,
@@ -491,9 +533,7 @@ class ErrorCatchingAsyncIterator:
                     'relaunch': str(exception.relaunch),
                 },
             )
-            raise stt.DataSpecErrorStatus(
-                (exception.relaunch, traceback.format_exc())
-            )
+            raise
         except Exception:
             stt.error(
                 dataspec=self.dataspec,
