@@ -1,8 +1,10 @@
+import os
 import typing as t
 
 import pyarrow as pa
 
 from sarus_data_spec import typing as st
+from sarus_data_spec.constants import CACHE_SCALAR_TASK, TO_PARQUET_TASK
 from sarus_data_spec.manager.asyncio.api.api_computation import ApiComputation
 from sarus_data_spec.manager.asyncio.base import BaseAsyncManager
 from sarus_data_spec.manager.asyncio.worker.arrow_computation import (
@@ -30,6 +32,13 @@ import sarus_data_spec.protobuf as sp
 import sarus_data_spec.status as stt
 import sarus_data_spec.storage.typing as storage_typing
 
+computation_timeout = os.environ.get(
+    'DELEGATING_COMPUTATION_TIMEOUT', default=600
+)
+computation_max_delay = os.environ.get(
+    'DELEGATING_COMPUTATION_MAX_DELAY', default=10
+)
+
 
 class DelegatingManager(BaseAsyncManager):
     """Manager that can compute a result locally or delegate a computation.
@@ -61,19 +70,17 @@ class DelegatingManager(BaseAsyncManager):
         super().__init__(storage, protobuf)
         self.remote_manager = remote_manager
         self.local_schema_computation = SchemaComputation(self)
-        self.local_to_arrow_computation = ToArrowComputation(self)
-        self.local_value_computation = ValueComputation(self)
-        self.local_to_parquet_computation = ToParquetComputation(
-            self, ToArrowComputation(self)
+        self.local_to_arrow_computation = ToArrowComputation(
+            self, ToParquetComputation(self)
         )
-        self.local_cache_scalar_computation = CacheScalarComputation(
-            self, ValueComputation(self)
+        self.local_value_computation = ValueComputation(
+            self, CacheScalarComputation(self)
         )
+        self.local_to_parquet_computation = ToParquetComputation(self)
+        self.local_cache_scalar_computation = CacheScalarComputation(self)
 
         # To define in subclasses
-        self.remote_to_parquet_computation: ApiComputation[
-            t.AsyncIterator[pa.RecordBatch]
-        ]
+        self.remote_to_parquet_computation: ApiComputation[None]
         self.remote_to_arrow_computation: ApiComputation[
             t.AsyncIterator[pa.RecordBatch]
         ]
@@ -82,31 +89,18 @@ class DelegatingManager(BaseAsyncManager):
 
     def dataspec_computation(self, dataspec: st.DataSpec) -> smt.Computation:
         """Return the computation for a Dataspec."""
-        is_cached = self.is_cached(dataspec)
         is_delegated = self.is_delegated(dataspec)
         proto = dataspec.prototype()
         if proto == sp.Dataset:
-            if is_cached:
-                if is_delegated:
-                    return self.remote_to_parquet_computation
-                else:
-                    return self.local_to_parquet_computation
+            if is_delegated:
+                return self.remote_to_arrow_computation
             else:
-                if is_delegated:
-                    return self.remote_to_arrow_computation
-                else:
-                    return self.local_to_arrow_computation
+                return self.local_to_arrow_computation
         else:
-            if is_cached:
-                if is_delegated:
-                    return self.remote_cache_scalar_computation
-                else:
-                    return self.local_cache_scalar_computation
+            if is_delegated:
+                return self.remote_value_computation
             else:
-                if is_delegated:
-                    return self.remote_value_computation
-                else:
-                    return self.local_value_computation
+                return self.local_value_computation
 
     def _delegate_manager_status(
         self, dataspec: st.DataSpec, task_name: str
@@ -146,6 +140,11 @@ class DelegatingManager(BaseAsyncManager):
             manager=self,
         )
 
+        # TODO think of a finer way to distinguish delegated tasks
+        if task_name in [CACHE_SCALAR_TASK, TO_PARQUET_TASK]:
+            # cannot delegate the caching
+            return local_status
+
         if not self.is_delegated(dataspec):
             # computation not delegated, return local status
             return local_status
@@ -165,28 +164,33 @@ class DelegatingManager(BaseAsyncManager):
 
         # copy delegate manager status as API status
         if delegate_stage.ready():
-            status = stt.ready(
+            status, _ = stt.ready(
                 dataspec=dataspec,
                 task=task_name,
                 manager=self,
                 properties=delegate_stage.properties(),
             )
         elif delegate_stage.error():
-            status = stt.error(
+            status, _ = stt.error(
                 dataspec=dataspec,
                 task=task_name,
                 manager=self,
                 properties=delegate_stage.properties(),
             )
         elif delegate_stage.processing():
-            status = stt.processing(
+            status, _ = stt.processing(
                 dataspec=dataspec,
                 task=task_name,
                 manager=self,
                 properties=delegate_stage.properties(),
             )
         elif delegate_stage.pending():
-            raise ValueError('Delegate manager statuses should not be pending')
+            status, _ = stt.pending(
+                dataspec=dataspec,
+                task=task_name,
+                manager=self,
+                properties=delegate_stage.properties(),
+            )
         else:
             raise ValueError('Delegate manager status not properly managed')
 
@@ -239,3 +243,12 @@ class DelegatingManager(BaseAsyncManager):
             return await TransformedDataset(dataset).schema()
         else:
             raise ValueError('Dataset should be transformed.')
+
+    def engine(self, uri: str) -> smt.sa_engine:
+        raise NotImplementedError
+
+    def computation_timeout(self, dataspec: st.DataSpec) -> int:
+        return int(computation_timeout)
+
+    def computation_max_delay(self, dataspec: st.DataSpec) -> int:
+        return int(computation_max_delay)
