@@ -12,39 +12,28 @@ from sarus_data_spec.constants import (
     CACHE_TYPE,
     ScalarCaching,
 )
-from sarus_data_spec.manager.asyncio.worker.value_computation import (
-    ValueComputation,
-)
 from sarus_data_spec.manager.asyncio.worker.worker_computation import (
-    TypedWorkerManager,
     WorkerComputation,
 )
+from sarus_data_spec.scalar import Scalar
 from sarus_data_spec.status import DataSpecErrorStatus, error, ready
 import sarus_data_spec.protobuf as sp
 
 logger = logging.getLogger(__name__)
 
 
-class CacheScalarComputation(WorkerComputation[t.Any]):
+class CacheScalarComputation(WorkerComputation[t.Tuple[str, str]]):
     """Class responsible for handling the caching
     in of a scalar. It wraps a ValueComputation to get the value."""
 
     task_name = CACHE_SCALAR_TASK
 
-    def __init__(
-        self,
-        manager: TypedWorkerManager,
-        value_computation: ValueComputation,
-    ) -> None:
-        super().__init__(manager)
-        self.value_computation = value_computation
-
     async def prepare(self, dataspec: st.DataSpec) -> None:
-
         logger.debug(f'STARTING CACHE_SCALAR {dataspec.uuid()}')
         try:
-            value = await self.value_computation.task_result(dataspec)
-
+            value = await self.manager().async_value_op(
+                scalar=t.cast(Scalar, dataspec)
+            )
             if isinstance(value, st.HasProtobuf):
                 properties = {
                     CACHE_PROTO: sp.to_base64(value.protobuf()),
@@ -59,7 +48,6 @@ class CacheScalarComputation(WorkerComputation[t.Any]):
                     pkl.dump(value, f)
 
         except DataSpecErrorStatus as exception:
-
             error(
                 dataspec=dataspec,
                 manager=self.manager(),
@@ -69,13 +57,9 @@ class CacheScalarComputation(WorkerComputation[t.Any]):
                     'relaunch': str(exception.relaunch),
                 },
             )
-
-            raise DataSpecErrorStatus(
-                relaunch=exception.relaunch, error_msg=traceback.format_exc()
-            )
+            raise
 
         except Exception:
-
             error(
                 dataspec=dataspec,
                 manager=self.manager(),
@@ -86,9 +70,7 @@ class CacheScalarComputation(WorkerComputation[t.Any]):
                 },
             )
 
-            raise DataSpecErrorStatus(
-                relaunch=False, error_msg=traceback.format_exc()
-            )
+            raise DataSpecErrorStatus((False, traceback.format_exc()))
         else:
             logger.debug(f'FINISHED CACHE_SCALAR {dataspec.uuid()}')
             ready(
@@ -103,16 +85,11 @@ class CacheScalarComputation(WorkerComputation[t.Any]):
         dataspec: st.DataSpec,
         properties: t.Mapping[str, str],
         **kwargs: t.Any,
-    ) -> t.Any:
+    ) -> t.Tuple[str, str]:
         """Reads the cache and returns the value."""
         if properties[CACHE_TYPE] == ScalarCaching.PICKLE.value:
-            with open(properties[CACHE_PATH], "rb") as f:
-                data = pkl.load(f)
-            return data
-
-        return sp.python_proto_factory(
-            properties[CACHE_PROTO], properties[CACHE_TYPE]
-        )
+            return properties[CACHE_TYPE], properties[CACHE_PATH]
+        return properties[CACHE_TYPE], properties[CACHE_PROTO]
 
     def cache_path(self, dataspec: st.DataSpec) -> str:
         """Returns the path where to cache the scalar."""
