@@ -20,7 +20,6 @@ class SchemaComputation(WorkerComputation[st.Schema]):
     task_name = SCHEMA_TASK
 
     async def prepare(self, dataspec: st.DataSpec) -> None:
-
         try:
             logger.debug(f'STARTED SCHEMA {dataspec.uuid()}')
             schema = await self.manager().async_schema_op(
@@ -37,9 +36,7 @@ class SchemaComputation(WorkerComputation[st.Schema]):
                     'relaunch': str(exception.relaunch),
                 },
             )
-            raise DataSpecErrorStatus(
-                relaunch=exception.relaunch, error_msg=traceback.format_exc()
-            )
+            raise
         except Exception:
             error(
                 dataspec=dataspec,
@@ -50,9 +47,7 @@ class SchemaComputation(WorkerComputation[st.Schema]):
                     'relaunch': str(False),
                 },
             )
-            raise DataSpecErrorStatus(
-                relaunch=False, error_msg=traceback.format_exc()
-            )
+            raise DataSpecErrorStatus((False, traceback.format_exc()))
         else:
             logger.debug(f'FINISHED SCHEMA {dataspec.uuid()}')
             ready(
