@@ -21,18 +21,21 @@ from sarus_data_spec.constants import (
     BIG_DATA_THRESHOLD,
     DATASET_N_BYTES,
     DATASET_N_LINES,
+    DATASET_TYPES,
     IS_BIG_DATA,
     IS_REMOTE,
     THRESHOLD_TYPE,
 )
+from sarus_data_spec.dataspec_rewriter.base import BaseDataspecRewriter
+from sarus_data_spec.dataspec_validator.base import BaseDataspecValidator
 from sarus_data_spec.protobuf.utilities import copy
 from sarus_data_spec.protobuf.utilities import json as utilities_json
 from sarus_data_spec.protobuf.utilities import serialize, type_name
-from sarus_data_spec.query_manager.base import BaseQueryManager
 from sarus_data_spec.scalar import Scalar
+import sarus_data_spec.dataspec_rewriter.typing as sdrt
+import sarus_data_spec.dataspec_validator.typing as sdvt
 import sarus_data_spec.manager.typing as manager_typing
 import sarus_data_spec.protobuf as sp
-import sarus_data_spec.query_manager.typing as sqmt
 import sarus_data_spec.status as stt
 import sarus_data_spec.storage.typing as storage_typing
 import sarus_data_spec.typing as st
@@ -50,10 +53,14 @@ class Base(manager_typing.Manager):
         self.storage().store(self)
         self._parquet_dir = os.path.expanduser('/tmp/sarus_dataset/')
         os.makedirs(self.parquet_dir(), exist_ok=True)
-        self._query_manager = BaseQueryManager(storage=storage)
+        self._dataspec_rewriter = BaseDataspecRewriter(storage=storage)
+        self._dataspec_validator = BaseDataspecValidator(storage=storage)
 
-    def query_manager(self) -> sqmt.QueryManager:
-        return self._query_manager
+    def dataspec_rewriter(self) -> sdrt.DataspecRewriter:
+        return self._dataspec_rewriter
+
+    def dataspec_validator(self) -> sdvt.DataspecValidator:
+        return self._dataspec_validator
 
     def parquet_dir(self) -> str:
         return self._parquet_dir
@@ -107,15 +114,6 @@ class Base(manager_typing.Manager):
     def storage(self) -> storage_typing.Storage:
         return self._storage
 
-    def schema(self, dataset: st.Dataset) -> st.Schema:
-        raise NotImplementedError
-
-    def marginals(self, dataset: st.Dataset) -> st.Marginals:
-        raise NotImplementedError
-
-    def size(self, dataset: st.Dataset) -> st.Size:
-        raise NotImplementedError
-
     def set_remote(self, dataspec: st.DataSpec) -> None:
         """Add an Attribute to tag the DataSpec as remotely fetched."""
         attach_properties(
@@ -132,6 +130,17 @@ class Base(manager_typing.Manager):
         is_remote_att = self.attribute(IS_REMOTE, dataspec)
         return is_remote_att is not None
 
+    def mock_value(
+        self,
+        transform: st.Transform,
+        *arguments: st.DataSpec,
+        **named_arguments: st.DataSpec,
+    ) -> t.Any:
+        """Compute the mock value of an external transform applied on
+        Dataspecs.
+        """
+        raise NotImplementedError
+
     def infer_output_type(
         self,
         transform: st.Transform,
@@ -143,7 +152,8 @@ class Base(manager_typing.Manager):
         def attach_nothing(ds: st.DataSpec) -> None:
             return
 
-        if transform.is_external() or transform.spec() in [
+        # Some internal transforms return a Scalar
+        if transform.spec() in [
             'protected_paths',
             'automatic_user_settings',
             'public_paths',
@@ -154,14 +164,29 @@ class Base(manager_typing.Manager):
             'derive_seed',
             'relationship_spec',
         ]:
-            # By default, results of non external transforms (e.g. join,
-            # sample) are Datasets and non external transforms are only applied
-            # to Datasets
             return sp.type_name(sp.Scalar), attach_nothing
 
-        return sp.type_name(sp.Dataset), attach_nothing
+        # Other internal transforms return a Dataset
+        if not transform.is_external():
+            return sp.type_name(sp.Dataset), attach_nothing
 
-    def bounds(self, dataset: st.Dataset) -> st.Bounds:
+        # Compute the mock value for external transforms
+        mock_value = self.mock_value(transform, *arguments, **named_arguments)
+
+        # Infer output types
+        dataspec_type = (
+            sp.type_name(sp.Dataset)
+            if type(mock_value) in DATASET_TYPES
+            else sp.type_name(sp.Scalar)
+        )
+
+        # Return the output type
+        return dataspec_type, attach_nothing
+
+    def schema(self, dataset: st.Dataset) -> st.Schema:
+        raise NotImplementedError
+
+    async def async_schema(self, dataset: st.Dataset) -> st.Schema:
         raise NotImplementedError
 
     def to_arrow(
@@ -169,9 +194,17 @@ class Base(manager_typing.Manager):
     ) -> t.Iterator[pa.RecordBatch]:
         raise NotImplementedError
 
+    async def async_to_arrow(
+        self, dataset: st.Dataset, batch_size: int
+    ) -> t.AsyncIterator[pa.RecordBatch]:
+        raise NotImplementedError
+
     def to_pandas(self, dataset: st.Dataset) -> pd.DataFrame:
         raise NotImplementedError
 
+    async def async_to_pandas(self, dataset: st.Dataset) -> pd.DataFrame:
+        raise NotImplementedError
+
     def to_parquet(self, dataset: st.Dataset) -> None:
         raise NotImplementedError
 
@@ -181,6 +214,11 @@ class Base(manager_typing.Manager):
     def to_tensorflow(self, dataset: st.Dataset) -> tf.data.Dataset:
         raise NotImplementedError
 
+    async def async_to_tensorflow(
+        self, dataset: st.Dataset
+    ) -> tf.data.Dataset:
+        raise NotImplementedError
+
     def value(self, scalar: st.Scalar) -> st.DataSpecValue:
         raise NotImplementedError
 
@@ -193,23 +231,19 @@ class Base(manager_typing.Manager):
     async def async_cache_scalar(self, scalar: st.Scalar) -> None:
         raise NotImplementedError
 
-    async def async_to_arrow(
-        self, dataset: st.Dataset, batch_size: int
-    ) -> t.AsyncIterator[pa.RecordBatch]:
+    def size(self, dataset: st.Dataset) -> st.Size:
         raise NotImplementedError
 
-    async def async_to_pandas(self, dataset: st.Dataset) -> pd.DataFrame:
+    async def async_size(self, dataset: st.Dataset) -> st.Size:
         raise NotImplementedError
 
-    async def async_to_tensorflow(
-        self, dataset: st.Dataset
-    ) -> tf.data.Dataset:
+    def bounds(self, dataset: st.Dataset) -> st.Bounds:
         raise NotImplementedError
 
     async def async_bounds(self, dataset: st.Dataset) -> st.Bounds:
         raise NotImplementedError
 
-    async def async_size(self, dataset: st.Dataset) -> st.Size:
+    def marginals(self, dataset: st.Dataset) -> st.Marginals:
         raise NotImplementedError
 
     async def async_marginals(self, dataset: st.Dataset) -> st.Marginals:
