@@ -5,7 +5,7 @@ import typing as t
 import pyarrow as pa
 
 from sarus_data_spec.manager.asyncio.utils import decoupled_async_iter
-import sarus_data_spec.query_manager.typing as sqmt
+import sarus_data_spec.dataspec_validator.typing as sdvt
 import sarus_data_spec.typing as st
 
 
@@ -58,7 +58,7 @@ class DatasetStaticChecker(DataspecStaticChecker):
         """Return a token if the output is PEP."""
         raise NotImplementedError
 
-    def pep_kind(self) -> sqmt.PEPKind:
+    def pep_kind(self) -> sdvt.PEPKind:
         raise NotImplementedError
 
 
