@@ -9,6 +9,7 @@ import pyarrow as pa
 
 from sarus_data_spec.arrow.schema import type_from_arrow_schema
 from sarus_data_spec.config import ROUTING
+from sarus_data_spec.dataspec_validator.typing import PEPKind
 from sarus_data_spec.manager.asyncio.utils import async_iter
 from sarus_data_spec.manager.ops.asyncio.base import (
     DatasetImplementation,
@@ -16,7 +17,6 @@ from sarus_data_spec.manager.ops.asyncio.base import (
     DataspecStaticChecker,
     ScalarImplementation,
 )
-from sarus_data_spec.query_manager.typing import PEPKind
 from sarus_data_spec.schema import schema as schema_builder
 from sarus_data_spec.transform import external, transform_id
 import sarus_data_spec.manager.typing as smt
@@ -26,23 +26,35 @@ import sarus_data_spec.typing as st
 
 from .protection_utils import (
     ExternalOpImplementation,
-    extract_data_from_pe,
-    pandas_merge_pe,
+    arguments_values,
+    arguments_values_and_protected_entity,
+    output_protected_entity,
+    to_pyarrow_table_with_protected_entity,
 )
 
 
 class ExternalScalarStaticChecker(DataspecStaticChecker):
     async def private_queries(self) -> t.List[st.PrivateQuery]:
         """Return the PrivateQueries summarizing DP characteristics."""
-        implementation = external_implementation(self.dataspec)
-        args, kwargs = external_arguments(self.dataspec)
+        transform = self.dataspec.transform()
+        ds_args, ds_kwargs = self.dataspec.parents()
+
+        implementation = external_implementation(transform)
+        args, kwargs = static_and_dynamic_arguments(
+            transform, *ds_args, **ds_kwargs
+        )
         return await implementation.private_queries(*args, **kwargs)
 
     def is_dp(self) -> bool:
         """Checks if the transform is DP and compatible with the arguments."""
-        op_implementation = external_implementation(self.dataspec)
-        args, kwargs = external_arguments(self.dataspec)
-        return op_implementation.is_dp(*args, **kwargs)
+        transform = self.dataspec.transform()
+        ds_args, ds_kwargs = self.dataspec.parents()
+
+        implementation = external_implementation(transform)
+        args, kwargs = static_and_dynamic_arguments(
+            transform, *ds_args, **ds_kwargs
+        )
+        return implementation.is_dp(*args, **kwargs)
 
     def is_dp_applicable(self, public_context: t.Collection[str]) -> bool:
         """Statically check if a DP transform is applicable in this position.
@@ -54,10 +66,15 @@ class ExternalScalarStaticChecker(DataspecStaticChecker):
             (i.e. same PEP token)
             - other dataspecs arguments are public
         """
-        op_implementation = external_implementation(self.dataspec)
-        args, kwargs = external_arguments(self.dataspec)
+        transform = self.dataspec.transform()
+        ds_args, ds_kwargs = self.dataspec.parents()
 
-        dp_implementation = op_implementation.dp_equivalent()
+        implementation = external_implementation(transform)
+        args, kwargs = static_and_dynamic_arguments(
+            transform, *ds_args, **ds_kwargs
+        )
+
+        dp_implementation = implementation.dp_equivalent()
         if dp_implementation is None or not dp_implementation.is_dp(
             *args, **kwargs
         ):
@@ -89,10 +106,9 @@ class ExternalScalarStaticChecker(DataspecStaticChecker):
 
     def dp_transform(self) -> t.Optional[st.Transform]:
         """Return the dataspec's DP equivalent transform if existing."""
-        op_implementation = external_implementation(self.dataspec)
-        py_args, py_kwargs, ds_args_pos = serialized_external_arguments(
-            self.dataspec
-        )
+        transform = self.dataspec.transform()
+        op_implementation = external_implementation(transform)
+        py_args, py_kwargs, ds_args_pos = static_arguments(transform)
 
         dp_implementation = op_implementation.dp_equivalent()
         if dp_implementation is None:
@@ -118,19 +134,20 @@ class ExternalDatasetStaticChecker(
 
     def pep_token(self, public_context: t.Collection[str]) -> t.Optional[str]:
         """Return the current dataspec's PEP token."""
-        op_implementation = external_implementation(self.dataspec)
-        transform_args, transform_kwargs = external_arguments(self.dataspec)
+        transform = self.dataspec.transform()
+        ds_args, ds_kwargs = self.dataspec.parents()
 
-        if len(op_implementation.allowed_pep_args) == 0:
+        implementation = external_implementation(transform)
+        args, kwargs = static_and_dynamic_arguments(
+            transform, *ds_args, **ds_kwargs
+        )
+
+        if len(implementation.allowed_pep_args) == 0:
             return None
 
-        pep_args, non_pep_args = group_by_pep(
-            op_implementation, *transform_args, **transform_kwargs
-        )
+        pep_args, non_pep_args = group_by_pep(implementation, *args, **kwargs)
 
-        pep_kind = op_implementation.pep_kind(
-            *transform_args, **transform_kwargs
-        )
+        pep_kind = implementation.pep_kind(*args, **kwargs)
         if pep_kind == PEPKind.NOT_PEP:
             return None
 
@@ -144,7 +161,7 @@ class ExternalDatasetStaticChecker(
             return None
 
         # The PEP arg combination should be allowed
-        if set(pep_args.keys()) not in op_implementation.allowed_pep_args:
+        if set(pep_args.keys()) not in implementation.allowed_pep_args:
             return None
 
         # All PEP tokens should be equal
@@ -193,82 +210,88 @@ class ExternalDatasetOp(DatasetImplementation):
     async def to_arrow(
         self, batch_size: int
     ) -> t.AsyncIterator[pa.RecordBatch]:
-        op_implementation = external_implementation(self.dataset)
-        transform_args, transform_kwargs = external_arguments(self.dataset)
+        transform = self.dataset.transform()
+        ds_args, ds_kwargs = self.dataset.parents()
 
-        data_args, data_kwargs, pe_candidates = await evaluate_arguments(
-            *transform_args, **transform_kwargs
+        implementation = external_implementation(transform)
+        args, kwargs = static_and_dynamic_arguments(
+            transform, *ds_args, **ds_kwargs
         )
-        static_checker = ExternalScalarStaticChecker(self.dataset)
-        if self.dataset.is_pep() or static_checker.is_dp():
-            # If we reach this part then there should be only one input PE
-            pe = next(iter(pe_candidates), None)
-            if pe is None:
-                raise ValueError(
-                    "The dataset was infered PEP but has no input PE"
-                )
-            # For now, PE in external ops are only viewed as pd.DataFrames
-            if not all([candidate.equals(pe) for candidate in pe_candidates]):
-                raise ValueError(
-                    "The dataset is PEP but has several differing"
-                    " input PE values"
-                )
 
+        static_checker = ExternalDatasetStaticChecker(self.dataset)
         if static_checker.is_dp():
-            # We also pass the PE for DP implementations
-            data = await op_implementation.call(
+            (
+                data_args,
+                data_kwargs,
+                pe,
+            ) = await arguments_values_and_protected_entity(*args, **kwargs)
+            result = await implementation.call(
                 *data_args, **data_kwargs, pe=pe, dataspec=self.dataset
             )
-        else:
-            data = await op_implementation.call(*data_args, **data_kwargs)
+            table = to_pyarrow_table(result)
 
-        if self.dataset.is_pep():
-            # We guarantee that the data.index is a reliable way to trace how
-            # the rows were rearranged
-            assert pe is not None
-            pe = pe.loc[data.index]
-        else:
-            pe = None
-
-        if isinstance(data, pd.DataFrame):
-            table = pandas_merge_pe(data, pe)
-            return async_iter(table.to_batches(max_chunksize=batch_size))
+        elif self.dataset.is_pep():
+            (
+                data_args,
+                data_kwargs,
+                pe,
+            ) = await arguments_values_and_protected_entity(*args, **kwargs)
+            result = await implementation.call(*data_args, **data_kwargs)
+            output_pe = output_protected_entity(pe, result)
+            table = to_pyarrow_table_with_protected_entity(result, output_pe)
 
         else:
-            raise TypeError(f"Cannot convert {type(data)} to Arrow batches.")
+            data_args, data_kwargs = await arguments_values(*args, **kwargs)
+            result = await implementation.call(*data_args, **data_kwargs)
+            table = to_pyarrow_table(result)
+
+        return async_iter(table.to_batches(max_chunksize=batch_size))
 
 
 class ExternalScalarOp(ScalarImplementation):
     async def value(self) -> t.Any:
-        op_implementation = external_implementation(self.scalar)
-        transform_args, transform_kwargs = external_arguments(self.scalar)
-
-        data_args, data_kwargs, pe_candidates = await evaluate_arguments(
-            *transform_args, **transform_kwargs
+        transform = self.scalar.transform()
+        ds_args, ds_kwargs = self.scalar.parents()
+        return await async_compute_external_value(
+            transform, *ds_args, **ds_kwargs
         )
 
-        static_checker = ExternalScalarStaticChecker(self.scalar)
-        if static_checker.is_dp():
-            # If we reach this part then there should be only one input PE
-            pe = next(iter(pe_candidates), None)
-            if pe is None:
-                raise ValueError(
-                    "The dataset was infered PEP but has no input PE"
-                )
-            # For now, PE in external ops are only viewed as pd.DataFrames
-            if not all([candidate.equals(pe) for candidate in pe_candidates]):
-                raise ValueError(
-                    "The dataset is PEP but has several differing"
-                    " input PE values"
-                )
-            # We also pass the PE for DP implementations
-            data = await op_implementation.call(
-                *data_args, **data_kwargs, pe=pe, dataspec=self.scalar
-            )
-        else:
-            data = await op_implementation.call(*data_args, **data_kwargs)
 
-        return data
+async def async_compute_external_value(
+    transform: st.Transform,
+    *ds_args: st.DataSpec,
+    **ds_kwargs: st.DataSpec,
+) -> t.Any:
+    """Compute the value of an external transform applied on Dataspecs.
+
+    This function computes the output value without manipulating the
+    corresponding Dataspec. This is useful when we need to have access
+    to the value of a Dataspec before its creation:
+      - for computing a Mock value and inferring if the result is
+        a Scalar or a Dataset.
+    """
+    implementation = external_implementation(transform)
+    args, kwargs = static_and_dynamic_arguments(
+        transform, *ds_args, **ds_kwargs
+    )
+
+    if implementation.is_dp(*args, **kwargs):
+        data = await implementation.call(*args, **kwargs)
+
+    else:
+        data_args, data_kwargs = await arguments_values(*args, **kwargs)
+        data = await implementation.call(*data_args, **data_kwargs)
+
+    return data
+
+
+def to_pyarrow_table(data: t.Any) -> pa.Table:
+    """Convert the result of an external transform to a Pyarrow Table."""
+    if not isinstance(data, pd.DataFrame):
+        raise TypeError(f"Cannot convert {type(data)} to Arrow batches.")
+
+    df = t.cast(pd.DataFrame, data)
+    return pa.Table.from_pandas(df)
 
 
 def group_by_pep(
@@ -318,14 +341,13 @@ def group_by_pep(
     return pep_args, non_pep_args
 
 
-def serialized_external_arguments(
-    dataspec: st.DataSpec,
+def static_arguments(
+    transform: st.Transform,
 ) -> t.Tuple[t.Dict[int, t.Any], t.Dict[str, t.Any], t.List[int]]:
     """Return the external arguments serialized in the protobuf."""
-    transform = dataspec.transform()
     assert transform and transform.is_external()
 
-    transform_spec = dataspec.transform().protobuf().spec
+    transform_spec = transform.protobuf().spec
     external_args = pkl.loads(transform_spec.external.named_arguments)
     py_args = external_args["py_args"]
     py_kwargs = external_args["py_kwargs"]
@@ -333,16 +355,14 @@ def serialized_external_arguments(
     return py_args, py_kwargs, ds_args_pos
 
 
-def external_arguments(
-    dataspec: st.DataSpec,
+def static_and_dynamic_arguments(
+    transform: st.Transform, *ds_args: st.DataSpec, **ds_kwargs: st.DataSpec
 ) -> t.Tuple[t.List[t.Any], t.Dict[str, t.Any]]:
     """Return all the external arguments.
 
-    This returns arguments serialized in the protobufs as well as Dataspecs.
+    This returns static arguments interleaved with Dataspecs.
     """
-    py_args, py_kwargs, ds_args_pos = serialized_external_arguments(dataspec)
-
-    ds_args, ds_kwargs = dataspec.parents()
+    py_args, py_kwargs, ds_args_pos = static_arguments(transform)
     pos_values = {pos: val for pos, val in zip(ds_args_pos, ds_args)}
     kwargs = {**py_kwargs, **ds_kwargs}
     pos_args = {**pos_values, **py_args}
@@ -351,13 +371,12 @@ def external_arguments(
 
 
 def external_implementation(
-    dataspec: st.DataSpec,
+    transform: st.Transform,
 ) -> smt.ExternalOpImplementation:
     """Return the implementation of an external op from a DataSpec.
 
     The mapping is done by the config file.
     """
-    transform = dataspec.transform()
     assert transform and transform.is_external()
     library, op_name = transform_id(transform).split(".")
     if op_name not in ROUTING["external"][library]:
@@ -381,27 +400,4 @@ def external_implementation(
             },
         )
 
-    return t.cast(smt.ExternalOpImplementation, op_implementation(dataspec))
-
-
-async def evaluate_arguments(
-    *args: t.Any, **kwargs: t.Any
-) -> t.Tuple[t.List[t.Any], t.Dict[str, t.Any], t.List[t.Any]]:
-    """Evaluate sarus dataspecs and extract the PE.
-
-    Compute the value of Dataspec arguments. Extract all the protections and
-    return them in a list.
-    """
-    data_pe_args = [await extract_data_from_pe(arg) for arg in args]
-    data_args = [data for data, _ in data_pe_args]
-    pe_args = [pe for _, pe in data_pe_args]
-
-    data_pe_kwargs = {
-        name: await extract_data_from_pe(arg) for name, arg in kwargs.items()
-    }
-    data_kwargs = {name: data for name, (data, _) in data_pe_kwargs.items()}
-    pe_kwargs = [pe for _, pe in data_pe_kwargs.values()]
-
-    pe_candidates = list(filter(lambda x: x is not None, pe_args + pe_kwargs))
-
-    return data_args, data_kwargs, pe_candidates
+    return t.cast(smt.ExternalOpImplementation, op_implementation())
