@@ -5,7 +5,7 @@ import numpy as np
 import pandas as pd
 import pandas._typing as pdt
 
-from sarus_data_spec.query_manager.typing import PEPKind
+from sarus_data_spec.dataspec_validator.typing import PEPKind
 import sarus_data_spec.typing as st
 
 from ..protection_utils import ExternalOpImplementation
