@@ -9,7 +9,11 @@ from sarus_data_spec.constants import MAX_MAX_MULT
 from sarus_data_spec.protobuf.utilities import unwrap
 import sarus_data_spec.typing as st
 
-from ..protection_utils import ExternalOpImplementation
+from ..protection_utils import (
+    ExternalOpImplementation,
+    arguments_values,
+    arguments_values_and_protected_entity,
+)
 
 logger = logging.getLogger(__name__)
 
@@ -58,14 +62,14 @@ NUMERIC_TYPES = ('integer', 'float', 'boolean')
 
 
 class pd_shape_dp(ExternalOpImplementation):
-    allowed_pep_args: t.List[t.Set[str]] = [{"dataframe"}]
+    allowed_pep_args: t.List[t.Set[str]] = [{"parent_ds"}]
     transform_id: str = "pandas.PD_SHAPE_DP"
 
     def is_dp(self, *args: t.Any, **kwargs: t.Any) -> bool:
         return True
 
     @staticmethod
-    async def static_privacy(  # type: ignore[override]
+    async def private_queries_and_task(  # type: ignore[override]
         parent_ds: st.Dataset, budget: t.Any, **kwargs: t.Any
     ) -> t.Tuple[t.List[st.PrivateQuery], st.Task]:
         """Return the PrivateQueries summarizing DP characteristics."""
@@ -111,11 +115,9 @@ class pd_shape_dp(ExternalOpImplementation):
 
     @staticmethod
     async def call(  # type: ignore[override]
-        dataframe: t.Any,
-        dataspec: st.DataSpec,
-        budget: t.Any,
-        seed: int,
-        pe: t.Any,
+        parent_ds: st.Dataset,
+        budget: st.Scalar,
+        seed: st.Scalar,
     ) -> t.Any:
         """Implementation of DP shape.
 
@@ -126,10 +128,17 @@ class pd_shape_dp(ExternalOpImplementation):
             - `seed`: an integer used to parametrize random number generators
             - `pe`: theprotected entity used by `sarus_statistics` primitives
         """
-        parent_ds = t.cast(st.Dataset, dataspec.parents()[0][0])
-        _, tasks = await pd_shape_dp.static_privacy(parent_ds, budget)
+        # Evaluate arguments
+        (dataframe, budget_value, seed_value), _ = await arguments_values(
+            parent_ds, budget, seed
+        )
 
-        epsilon = budget[0].epsilon
+        # Get QB task parametrization
+        _, tasks = await pd_shape_dp.private_queries_and_task(
+            parent_ds, budget_value
+        )
+
+        epsilon = budget_value[0].epsilon
         n_rows, n_cols = dataframe.shape
 
         # Compute DP value
@@ -137,7 +146,7 @@ class pd_shape_dp(ExternalOpImplementation):
         max_mult_task, shape_task = t.cast(MaxMultiplicity, tasks[0]), t.cast(
             GenericTask, tasks[1]
         )
-        random_generator = np.random.default_rng(abs(seed))
+        random_generator = np.random.default_rng(abs(seed_value))
         max_mul = MaxMultiplicityOp(
             parent_ds,
             epsilon,  # parameter for quantiles
@@ -158,12 +167,12 @@ class pd_shape_dp(ExternalOpImplementation):
 
 
 class pd_sum_dp(ExternalOpImplementation):
-    allowed_pep_args: t.List[t.Set[str]] = [{"dataframe"}]
+    allowed_pep_args: t.List[t.Set[str]] = [{"parent_ds"}]
     transform_id: str = "pandas.PD_SUM_DP"
 
     def is_dp(  # type: ignore[override]
         self,
-        dataframe: t.Any,
+        parent_ds: st.Dataset,
         axis: t.Optional[pdt.Axis] = None,
         skipna: bool = True,
         level: t.Optional[pdt.Level] = None,
@@ -176,7 +185,7 @@ class pd_sum_dp(ExternalOpImplementation):
         return (axis == 0) and (numeric_only is True)
 
     @staticmethod
-    async def static_privacy(  # type: ignore[override]
+    async def private_queries_and_task(  # type: ignore[override]
         parent_ds: st.Dataset,
         budget: t.Any,
         **kwargs: t.Any,
@@ -234,38 +243,39 @@ class pd_sum_dp(ExternalOpImplementation):
 
     @staticmethod
     async def call(  # type: ignore[override]
-        dataframe: t.Any,
-        dataspec: st.DataSpec,
+        parent_ds: st.Dataset,
         axis: t.Optional[pdt.Axis] = None,
         skipna: bool = True,
         level: t.Optional[pdt.Level] = None,
         numeric_only: t.Optional[bool] = None,
         min_count: int = 0,
-        budget: t.Any = None,
-        seed: t.Optional[int] = None,
-        pe: pd.DataFrame = None,
+        budget: t.Optional[st.Scalar] = None,
+        seed: t.Optional[st.Scalar] = None,
         **kwargs: t.Any,
     ) -> t.Any:
+        # Evaluate arguments
+        (dataframe, budget_value, seed_value), _ = await arguments_values(
+            parent_ds, budget, seed
+        )
+
         assert type(dataframe) in [pd.DataFrame, pd.Series]
-        assert pe.shape[0] == dataframe.shape[0]
         assert numeric_only
         assert axis == 0
         assert level is None
         assert min_count == 0
-        assert seed
-        assert budget
+        assert seed_value
+        assert budget_value
 
-        parent_ds = t.cast(st.Dataset, dataspec.parents()[0][0])
-        _, tasks = await pd_sum_dp.static_privacy(
+        _, tasks = await pd_sum_dp.private_queries_and_task(
             parent_ds=parent_ds,
-            budget=budget,
+            budget=budget_value,
         )
 
-        epsilon = budget[0].epsilon
+        epsilon = budget_value[0].epsilon
         # Compute DP value
         tasks = [unwrap(subtask) for subtask in tasks.subtasks]
         max_mult_task, sum_tasks = tasks[0], tasks[1:]
-        random_generator = np.random.default_rng(abs(seed))
+        random_generator = np.random.default_rng(abs(seed_value))
         max_mul = MaxMultiplicityOp(
             parent_ds,
             epsilon,  # parameter for quantiles
@@ -316,12 +326,12 @@ class pd_sum_dp(ExternalOpImplementation):
 
 
 class pd_mean_dp(ExternalOpImplementation):
-    allowed_pep_args: t.List[t.Set[str]] = [{"dataframe"}]
+    allowed_pep_args: t.List[t.Set[str]] = [{"parent_ds"}]
     transform_id: str = "pandas.PD_MEAN_DP"
 
     def is_dp(  # type: ignore[override]
         self,
-        dataframe: t.Any,
+        parent_ds: st.Dataset,
         axis: t.Optional[pdt.Axis] = 0,
         skipna: bool = True,
         level: t.Optional[pdt.Level] = None,
@@ -333,7 +343,7 @@ class pd_mean_dp(ExternalOpImplementation):
         return axis == 0 and numeric_only is True
 
     @staticmethod
-    async def static_privacy(  # type: ignore[override]
+    async def private_queries_and_task(  # type: ignore[override]
         parent_ds: st.Dataset,
         budget: t.Any,
         **kwargs: t.Any,
@@ -389,36 +399,42 @@ class pd_mean_dp(ExternalOpImplementation):
 
     @staticmethod
     async def call(  # type: ignore[override]
-        dataframe: t.Any,
-        dataspec: st.DataSpec,
+        parent_ds: st.Dataset,
         axis: t.Optional[pdt.Axis] = 0,
         skipna: bool = True,
         level: t.Optional[pdt.Level] = None,
         numeric_only: t.Optional[bool] = None,
-        budget: t.Any = None,
-        seed: t.Optional[int] = None,
-        pe: pd.DataFrame = None,
+        budget: t.Optional[st.Scalar] = None,
+        seed: t.Optional[st.Scalar] = None,
         **kwargs: t.Any,
     ) -> t.Any:
+        # Evaluate arguments
+        (
+            (dataframe, budget_value, seed_value),
+            _,
+            pe,
+        ) = await arguments_values_and_protected_entity(
+            parent_ds, budget, seed
+        )
+
         assert type(dataframe) in [pd.DataFrame, pd.Series]
         assert pe.shape[0] == dataframe.shape[0]
         assert numeric_only
         assert axis == 0
         assert level is None
-        assert seed
-        assert budget
+        assert seed_value
+        assert budget_value
 
-        parent_ds = t.cast(st.Dataset, dataspec.parents()[0][0])
-        _, tasks = await pd_mean_dp.static_privacy(
+        _, tasks = await pd_mean_dp.private_queries_and_task(
             parent_ds=parent_ds,
-            budget=budget,
+            budget=budget_value,
         )
         tasks = [unwrap(subtask) for subtask in tasks.subtasks]
         max_mult_task, mean_tasks = tasks[0], tasks[1:]
 
         # Compute DP value
-        random_generator = np.random.default_rng(abs(seed))
-        epsilon = budget[0].epsilon
+        random_generator = np.random.default_rng(abs(seed_value))
+        epsilon = budget_value[0].epsilon
         max_mul = MaxMultiplicityOp(
             parent_ds,
             epsilon,  # parameter for quantiles
@@ -466,12 +482,12 @@ class pd_mean_dp(ExternalOpImplementation):
 
 
 class pd_median_dp(ExternalOpImplementation):
-    allowed_pep_args: t.List[t.Set[str]] = [{"dataframe"}]
+    allowed_pep_args: t.List[t.Set[str]] = [{"parent_ds"}]
     transform_id: str = "pandas.PD_MEDIAN_DP"
 
     def is_dp(  # type: ignore[override]
         self,
-        dataframe: t.Any,
+        parent_ds: st.Dataset,
         axis: t.Optional[pdt.Axis] = 0,
         skipna: bool = True,
         level: t.Optional[pdt.Level] = None,
@@ -483,7 +499,7 @@ class pd_median_dp(ExternalOpImplementation):
         return axis == 0 and numeric_only is True
 
     @staticmethod
-    async def static_privacy(  # type: ignore[override]
+    async def private_queries_and_task(  # type: ignore[override]
         parent_ds: st.Dataset,
         budget: t.Any,
         **kwargs: t.Any,
@@ -540,35 +556,41 @@ class pd_median_dp(ExternalOpImplementation):
 
     @staticmethod
     async def call(  # type: ignore[override]
-        dataframe: t.Any,
-        dataspec: st.DataSpec,
+        parent_ds: st.Dataset,
         axis: t.Optional[pdt.Axis] = 0,
         skipna: bool = True,
         level: t.Optional[pdt.Level] = None,
         numeric_only: t.Optional[bool] = None,
-        budget: t.Any = None,
-        seed: t.Optional[int] = None,
-        pe: pd.DataFrame = None,
+        budget: t.Optional[st.Scalar] = None,
+        seed: t.Optional[st.Scalar] = None,
         **kwargs: t.Any,
     ) -> t.Any:
+        # Evaluate arguments
+        (
+            (dataframe, budget_value, seed_value),
+            _,
+            pe,
+        ) = await arguments_values_and_protected_entity(
+            parent_ds, budget, seed
+        )
+
         assert type(dataframe) in [pd.DataFrame, pd.Series]
         assert pe.shape[0] == dataframe.shape[0]
         assert numeric_only
         assert axis == 0
         assert level is None
-        assert seed
-        assert budget
+        assert seed_value
+        assert budget_value
 
-        parent_ds = t.cast(st.Dataset, dataspec.parents()[0][0])
-        _, tasks = await pd_median_dp.static_privacy(
+        _, tasks = await pd_median_dp.private_queries_and_task(
             parent_ds=parent_ds,
-            budget=budget,
+            budget=budget_value,
         )
         tasks = [unwrap(subtask) for subtask in tasks.subtasks]
         max_mult_task, median_tasks = tasks[0], tasks[1:]
 
-        random_generator = np.random.default_rng(abs(seed))
-        epsilon = budget[0].epsilon
+        random_generator = np.random.default_rng(abs(seed_value))
+        epsilon = budget_value[0].epsilon
 
         # Compute DP value
         max_mul = MaxMultiplicityOp(
@@ -618,12 +640,12 @@ class pd_median_dp(ExternalOpImplementation):
 
 
 class pd_std_dp(ExternalOpImplementation):
-    allowed_pep_args: t.List[t.Set[str]] = [{"dataframe"}]
+    allowed_pep_args: t.List[t.Set[str]] = [{"parent_ds"}]
     transform_id: str = "pandas.PD_STD_DP"
 
     def is_dp(  # type: ignore[override]
         self,
-        dataframe: t.Any,
+        parent_ds: t.Any,
         axis: t.Optional[pdt.Axis] = 0,
         skipna: bool = True,
         level: t.Optional[pdt.Level] = None,
@@ -636,7 +658,7 @@ class pd_std_dp(ExternalOpImplementation):
         return axis == 0 and numeric_only is True
 
     @staticmethod
-    async def static_privacy(  # type: ignore[override]
+    async def private_queries_and_task(  # type: ignore[override]
         parent_ds: st.Dataset,
         budget: t.Any,
         **kwargs: t.Any,
@@ -693,36 +715,42 @@ class pd_std_dp(ExternalOpImplementation):
 
     @staticmethod
     async def call(  # type: ignore[override]
-        dataframe: t.Any,
-        dataspec: st.DataSpec,
+        parent_ds: st.Dataset,
         axis: t.Optional[pdt.Axis] = 0,
         skipna: bool = True,
         level: t.Optional[pdt.Level] = None,
         numeric_only: t.Optional[bool] = None,
         ddof: int = 1,
-        budget: t.Any = None,
-        seed: t.Optional[int] = None,
-        pe: pd.DataFrame = None,
+        budget: t.Optional[st.Scalar] = None,
+        seed: t.Optional[st.Scalar] = None,
         **kwargs: t.Any,
     ) -> t.Any:
+        # Evaluate arguments
+        (
+            (dataframe, budget_value, seed_value),
+            _,
+            pe,
+        ) = await arguments_values_and_protected_entity(
+            parent_ds, budget, seed
+        )
+
         assert type(dataframe) in [pd.DataFrame, pd.Series]
         assert pe.shape[0] == dataframe.shape[0]
         assert numeric_only
         assert axis == 0
         assert level is None
-        assert seed
-        assert budget
+        assert seed_value
+        assert budget_value
 
-        parent_ds = t.cast(st.Dataset, dataspec.parents()[0][0])
-        _, tasks = await pd_std_dp.static_privacy(
+        _, tasks = await pd_std_dp.private_queries_and_task(
             parent_ds=parent_ds,
-            budget=budget,
+            budget=budget_value,
         )
         tasks = [unwrap(subtask) for subtask in tasks.subtasks]
         max_mult_task, std_tasks = tasks[0], tasks[1:]
 
-        epsilon = budget[0].epsilon
-        random_generator = np.random.default_rng(abs(seed))
+        epsilon = budget_value[0].epsilon
+        random_generator = np.random.default_rng(abs(seed_value))
 
         # Compute DP value
         max_mul = MaxMultiplicityOp(
