@@ -167,7 +167,7 @@ class DifferentiatedSample(StandardDatasetImplementation):
         size_dict = await differentiated_sampling_sizes(self.dataset)
 
         indices_to_take = sample_indices_from_array(
-            array=struct_arr.field(DATA),
+            array=struct_arr.flatten()[struct_arr.type.get_field_index(DATA)],
             stat=previous_size.statistics(),
             selected_indices=pa.array(
                 np.linspace(0, len(struct_arr) - 1, len(struct_arr), dtype=int)
@@ -211,6 +211,7 @@ def sample_indices_from_array(
 
             new_indices = []
             for field, field_stat in fields.items():
+                index_element = self.batch_array.type.get_field_index(field)
                 filter_idex = pa.array(
                     np.equal(
                         self.batch_array.field('field_selected'),
@@ -218,7 +219,9 @@ def sample_indices_from_array(
                     )
                 )
                 updated_indices = sample_indices_from_array(
-                    self.batch_array.filter(filter_idex).field(field),
+                    self.batch_array.filter(filter_idex).flatten()[
+                        index_element
+                    ],
                     field_stat,
                     self.indices.filter(filter_idex),
                     new_size_dict,
