@@ -4,7 +4,7 @@ import pyarrow as pa
 
 from sarus_data_spec.arrow.array import convert_record_batch
 from sarus_data_spec.bounds import bounds as bounds_builder
-from sarus_data_spec.constants import DATASET_SLUGNAME
+from sarus_data_spec.constants import DATA, DATASET_SLUGNAME
 from sarus_data_spec.dataset import Dataset
 from sarus_data_spec.manager.ops.asyncio.processor.standard.standard_op import (  # noqa: E501
     StandardDatasetImplementation,
@@ -30,8 +30,8 @@ class GetItemStaticChecker(StandardDatasetStaticChecker):
         new_type = sub_types[0]
         # TODO: update foreign_keys/primary_keys in the type
         previous_fields = parent_schema.type().children()
-        if 'data' in previous_fields.keys():
-            previous_fields['data'] = new_type
+        if DATA in previous_fields.keys():
+            previous_fields[DATA] = new_type
             new_type = sdt.Struct(fields=previous_fields)
         return schema(
             self.dataset,
@@ -59,15 +59,16 @@ class GetItem(StandardDatasetImplementation):
                 record_batch=batch, _type=parent_schema.type()
             )
             # VERY UGLY SHOULD BE REMOVED WHEN WE HAVE PROTECTED TYPE
-            if 'data' in parent_schema.type().children():
+            if DATA in parent_schema.type().children():
                 old_arrays = array.flatten()
-                array = array.field('data')
+                idx_data = array.type.get_field_index(DATA)
+                array = old_arrays[idx_data]
                 updated_array = get_items(
                     _type=parent_schema.data_type(),
                     array=array,
                     path=path,
                 )
-                old_arrays[0] = updated_array
+                old_arrays[idx_data] = updated_array
                 return pa.StructArray.from_arrays(
                     old_arrays,
                     names=list(parent_schema.type().children().keys()),
@@ -134,8 +135,9 @@ def get_items(array: pa.Array, path: st.Path, _type: st.Type) -> pa.Array:
         ) -> None:
             if len(path.sub_paths()) > 0:
                 sub_path = path.sub_paths()[0]
+                idx = array.type.get_field_index(sub_path.label())
                 self.batch_array = get_items(
-                    array=array.field(sub_path.label()),
+                    array=array.flatten()[idx],
                     path=sub_path,
                     _type=fields[sub_path.label()],
                 )
@@ -155,7 +157,8 @@ def get_items(array: pa.Array, path: st.Path, _type: st.Type) -> pa.Array:
             name: t.Optional[str] = None,
             properties: t.Optional[t.Mapping[str, str]] = None,
         ) -> None:
-            array = self.batch_array.field(path.label())
+            idx = self.batch_array.type.get_field_index(path.label())
+            array = self.batch_array.flatten()(idx)
             if len(path.sub_paths()) == 0:
                 self.batch_array = array
             else:
@@ -173,8 +176,9 @@ def get_items(array: pa.Array, path: st.Path, _type: st.Type) -> pa.Array:
                 self.batch_array = array
             else:
                 sub_path = path.sub_paths()[0]
+                idx = array.type.get_field_index(sub_path.label())
                 self.batch_array = get_items(
-                    array=array.field(sub_path.label()),
+                    array=array.flatten()[idx],
                     path=sub_path,
                     _type=fields[sub_path.label()],
                 )
