@@ -125,7 +125,11 @@ class StandardDatasetImplementation(DatasetImplementation):
         async for batch in async_iterator:
             new_array = await func_to_apply(batch)
             if len(new_array) == batch_size and global_array is None:
-                yield pa.RecordBatch.from_struct_array(new_array)
+                yield pa.RecordBatch.from_struct_array(
+                    new_array.take(
+                        np.linspace(0, batch_size - 1, batch_size, dtype=int)
+                    )
+                )
             elif global_array is not None:
                 global_array = pa.concat_arrays([global_array, new_array])
                 if len(global_array) < batch_size:
