@@ -28,7 +28,7 @@ def select_rows(
 
     class RowsSelector(st.TypeVisitor):
         batch_array: pa.Array = array
-        filter_indices = pa.array([True for _ in range(len(array))])
+        filter_indices: pa.Array
 
         def Null(
             self, properties: t.Optional[t.Mapping[str, str]] = None
@@ -297,7 +297,9 @@ def select_rows(
 
             # now get sub-levels filter_indices
             arrays = [
-                self.batch_array.field(field_name)
+                self.batch_array.flatten()[
+                    self.batch_array.type.get_field_index(field_name)
+                ]
                 for i, (field_name, field_type) in enumerate(fields.items())
             ]
             arrays_indices = [
@@ -420,12 +422,14 @@ def select_columns(_type: st.Type, array: pa.Array) -> pa.Array:
             name: t.Optional[str] = None,
             properties: t.Optional[t.Mapping[str, str]] = None,
         ) -> None:
-            assert len(fields) == self.batch_array.type.num_children - 1
+            assert len(fields) == self.batch_array.type.num_fields - 1
             # -1 because field_selected is not in the fields
             arrays = [
                 select_columns(
                     field_type,
-                    self.batch_array.field(field_name).filter(
+                    self.batch_array.flatten()[
+                        self.batch_array.type.get_field_index(field_name)
+                    ].filter(
                         pa.array(
                             self.batch_array.field('field_selected').to_numpy(
                                 zero_copy_only=False
