@@ -28,6 +28,12 @@ try:
     )
 except ModuleNotFoundError:
     logger.info("CSV package not found, source CSV operations not available.")
+try:
+    from sarus_data_spec.manager.ops.asyncio.processor.standard.synthetic import (  # noqa: E501
+        SyntheticModel,
+    )
+except ModuleNotFoundError:
+    logger.info("Synthetic generation not available")
 
 
 def get_scalar_op(scalar: st.Scalar) -> t.Type[ScalarImplementation]:
@@ -37,6 +43,8 @@ def get_scalar_op(scalar: st.Scalar) -> t.Type[ScalarImplementation]:
         return RandomSeed
     elif scalar.is_privacy_params():
         return PrivacyParams
+    elif scalar.is_synthetic_model():
+        return SyntheticModel
     else:
         raise NotImplementedError(f"Source scalar for {scalar}")
 
