@@ -24,8 +24,9 @@ except ModuleNotFoundError:
 import warnings
 
 from sarus_data_spec.storage.typing import HasStorage
+import sarus_data_spec.dataspec_rewriter.typing as sdrt
+import sarus_data_spec.dataspec_validator.typing as sdvt
 import sarus_data_spec.protobuf as sp
-import sarus_data_spec.query_manager.typing as sqmt
 import sarus_data_spec.typing as st
 
 try:
@@ -170,7 +171,10 @@ class Manager(st.Referrable[sp.Manager], HasStorage, Protocol):
     ) -> t.Optional[st.Status]:
         ...
 
-    def query_manager(self) -> sqmt.QueryManager:
+    def dataspec_rewriter(self) -> sdrt.DataspecRewriter:
+        ...
+
+    def dataspec_validator(self) -> sdvt.DataspecValidator:
         ...
 
     def is_remote(self, dataspec: st.DataSpec) -> bool:
@@ -271,6 +275,16 @@ class Manager(st.Referrable[sp.Manager], HasStorage, Protocol):
     def engine(self, uri: str) -> sa_engine:
         ...
 
+    def mock_value(
+        self,
+        transform: st.Transform,
+        *arguments: st.DataSpec,
+        **named_arguments: st.DataSpec,
+    ) -> t.Any:
+        """Compute the mock value of an external transform applied on
+        Dataspecs.
+        """
+
 
 @runtime_checkable
 class HasManager(Protocol):
@@ -365,7 +379,7 @@ class ExternalOpImplementation(t.Protocol):
             - `pe` the protected entity information of each row
         """
 
-    def pep_kind(self, *args: t.Any, **kwargs: t.Any) -> sqmt.PEPKind:
+    def pep_kind(self, *args: t.Any, **kwargs: t.Any) -> sdvt.PEPKind:
         """Return the PEP properties of the transform.
 
         It takes the transform arguments as input because it can depend on some
