@@ -54,11 +54,13 @@ class Path(Base[sp.Path]):
 
         final_sub_paths = []
         for sub_path in select_path.sub_paths():
+            should_add = False
             for available_sub_path in self.sub_paths():
                 if available_sub_path.label() == sub_path.label():
+                    should_add = True
                     break
-
-            final_sub_paths.extend(available_sub_path.select(sub_path))
+            if should_add:
+                final_sub_paths.extend(available_sub_path.select(sub_path))
 
         return final_sub_paths
 
