@@ -142,8 +142,9 @@ class Statistics(Base[sp.Statistics]):
                     for field in self._protobuf.struct.fields
                 },
                 self._protobuf.struct.size,
-                name=self._protobuf.struct.name,
+                name=self._protobuf.name,
                 multiplicity=self._protobuf.struct.multiplicity,
+                properties=self._protobuf.properties,
             ),
             'union': lambda: visitor.Union(
                 {
@@ -151,8 +152,9 @@ class Statistics(Base[sp.Statistics]):
                     for field in self._protobuf.union.fields
                 },
                 size=self._protobuf.union.size,
-                name=self._protobuf.union.name,
+                name=self._protobuf.name,
                 multiplicity=self._protobuf.union.multiplicity,
+                properties=self._protobuf.properties,
             ),
             'optional': lambda: visitor.Optional(
                 Statistics(self._protobuf.optional.statistics),
@@ -296,7 +298,6 @@ class Statistics(Base[sp.Statistics]):
                 name: t.Optional[str] = None,
                 properties: t.Optional[t.Mapping[str, str]] = None,
             ) -> None:
-
                 result = []
                 for sub_path in path.sub_paths():
                     result.extend(
@@ -725,6 +726,7 @@ class Distribution(Base[sp.Distribution]):
 
 # A few builders
 
+
 # Distributions
 def Integer_Distribution(
     min_value: t.Optional[int] = None,
@@ -954,7 +956,6 @@ def Float(
     values: t.Optional[t.List[float]] = None,
     properties: t.Optional[t.Mapping[str, str]] = None,
 ) -> Statistics:
-
     return Statistics(
         sp.Statistics(
             name='Float',
@@ -1018,7 +1019,6 @@ def Struct(
     name: t.Optional[str] = None,
     properties: t.Optional[t.Mapping[str, str]] = None,
 ) -> Statistics:
-
     return Statistics(
         sp.Statistics(
             name='Struct' if name is None else name,
@@ -1093,7 +1093,6 @@ def List(
     values: t.Optional[t.List[int]] = None,
     properties: t.Optional[t.Mapping[str, str]] = None,
 ) -> Statistics:
-
     return Statistics(
         sp.Statistics(
             name=name,
@@ -1124,7 +1123,6 @@ def Array(
     values: t.Optional[t.List[t.List[float]]] = None,
     properties: t.Optional[t.Mapping[str, str]] = None,
 ) -> Statistics:
-
     if min_values is None:
         min_values = []
     if max_values is None:
