@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 from datetime import datetime
+from functools import partial
 from typing import Callable, List, Mapping, Optional, Type, cast
 import logging
 import typing as t
@@ -46,7 +47,7 @@ class Status(Referring[sp.Status]):
         self,
         task_stages: Optional[Mapping[str, st.Stage]] = None,
         properties: Optional[Mapping[str, str]] = None,
-    ) -> Status:
+    ) -> t.Tuple[Status, bool]:
         """
         This method apply an atomic update
         to the last version of the status. The update
@@ -54,78 +55,53 @@ class Status(Referring[sp.Status]):
         to the new stage is a regression (see method
         transitions_allowed)
         """
-
-        def update_last(
-            status: st.Referring[ProtobufWithUUIDAndDatetime],
-        ) -> t.Tuple[st.Referring[ProtobufWithUUIDAndDatetime], bool]:
-            """Returns whether the update should be done, ie transitions
-            are respected and the status that can be updated in case"""
-            # We copy the content of the current status
-            proto = cast(Status, status).protobuf()
-            # We update its timestamp and properties
-            proto.datetime = datetime.now().isoformat()
-            if properties is not None:
-                proto.properties.update(properties)
-                # Protobuf implementation of the merge
-                # does not work as expected on maps...
-            if task_stages is not None:
-                for task in task_stages:
-                    stage = task_stages[task].protobuf()
-                    if task in proto.task_stages and proto.task_stages[
-                        task
-                    ].WhichOneof('stage') == task_stages[
-                        task
-                    ].protobuf().WhichOneof(
-                        'stage'
-                    ):
-                        proto.task_stages[task].properties.update(
-                            stage.properties
-                        )
-                    else:
-                        if transition_allowed(
-                            existing_stage=proto.task_stages.get(task, None),
-                            new_stage=stage,
-                            task_name=task,
-                        ):
-                            proto.task_stages[task].CopyFrom(stage)
-                        else:
-                            return status, False
-            # And return a new UNSTORED data spec object
-            return Status(proto, store=False), True
-
         # Now update the last version of this status
-        return cast(
-            Status,
-            self.storage().update_last_referring(
-                self.referred(), self.type_name(), update_last
+        new_status, is_updated = self.storage().update_referring_with_datetime(
+            self.referred(),
+            self.type_name(),
+            partial(
+                update_last, task_stages=task_stages, properties=properties
             ),
         )
+        return t.cast(Status, new_status), is_updated
 
-    def clear_task(self, task: str) -> t.Optional[Status]:
+    def clear_task(self, task: str) -> t.Tuple[Status, bool]:
         """Creates a new status removing the task specified.
         To remove the task specified, the stage must be either
         ready or error (a static situation in which no one
         is processing the task).
         If the task does not exist, nothing is created"""
         # We copy the content of the current status
-        proto = self.protobuf()
-        # We update its timestamp and properties
-        proto.datetime = datetime.now().isoformat()
-        stage = proto.task_stages.pop(task)
-        if stage.WhichOneof('stage') is None:
-            return None
-        # we can create a new empty status only
-        # if the current stage is static ie ready
-        # (for example when removing cache) or
-        # error (relaunch after timeout)
-        else:
-            can_be_cleared = transition_allowed(
-                existing_stage=stage, new_stage=None, task_name=task
-            )
-            if can_be_cleared:
-                # create a new data spec object
-                return Status(proto)
-            return None
+
+        def clear_update(
+            status: st.Referring[ProtobufWithUUIDAndDatetime],
+        ) -> t.Tuple[st.Referring[ProtobufWithUUIDAndDatetime], bool]:
+
+            status = t.cast(Status, status)
+            proto = status.protobuf()
+            # We update its timestamp and properties
+            proto.datetime = datetime.now().isoformat()
+            stage = proto.task_stages.pop(task)
+            if stage.WhichOneof('stage') is None:
+                return status, False
+
+            # we can create a new empty status only
+            # if the current stage is static ie ready
+            # (for example when removing cache) or
+            # error (relaunch after timeout)
+            else:
+                can_be_cleared = transition_allowed(
+                    existing_stage=stage, new_stage=None, task_name=task
+                )
+                if can_be_cleared:
+                    # create a new data spec object
+                    return Status(proto, store=False), True
+                return status, False
+
+        new_status, is_updated = self.storage().update_referring_with_datetime(
+            self.referred(), self.type_name(), clear_update
+        )
+        return cast(Status, new_status), is_updated
 
     def task(self, task: str) -> t.Optional[Stage]:
         stage = self.protobuf().task_stages.get(task, None)
@@ -228,16 +204,16 @@ def transition_allowed(
         'pending': ['processing', 'ready', 'error'],
         'processing': ['ready', 'error'],
         'ready': ['error', None],
-        'error': [None, 'ready'],  # the transition error to ready can occur
+        'error': [None, 'ready'],
+        None: ['pending', 'processing', 'ready', 'error']
+        # the transition error to ready can occur
         # if a long task is processed, meanwhile another worker sets an error
         # because it timeouts after waiting, then the first worker finishes
     }
     if new_stage is not None:
         if (
             new_stage.WhichOneof('stage')  # type:ignore
-            in allowed_transitions[
-                existing_stage.WhichOneof('stage')  # type:ignore
-            ]
+            in allowed_transitions[existing_stage.WhichOneof('stage')]
         ):
             return True
         else:
@@ -252,10 +228,8 @@ def transition_allowed(
 
     else:
         if (
-            new_stage  # type:ignore
-            in allowed_transitions[
-                existing_stage.WhichOneof('stage')  # type:ignore
-            ]
+            new_stage  # type: ignore
+            in allowed_transitions[existing_stage.WhichOneof('stage')]
         ):
             return True
 
@@ -276,7 +250,7 @@ def status(
     task_stages: Optional[Mapping[str, st.Stage]] = None,
     properties: Optional[Mapping[str, str]] = None,
     manager: Optional[Manager] = None,
-) -> st.Status:
+) -> t.Tuple[st.Status, bool]:
     """A builder to ease the construction of a status
     - dataspec: is the dataspec, the status is added to
     - manager: is which manager, if None,
@@ -288,7 +262,7 @@ def status(
     # and manager simply create an updated version of it
     last = last_status(dataspec=dataspec, manager=manager)
     if last is None:
-        return Status(
+        status = Status(
             sp.Status(
                 dataspec=dataspec.uuid(),
                 manager=manager.uuid(),
@@ -299,8 +273,19 @@ def status(
                     task: task_stages[task].protobuf() for task in task_stages
                 },
                 properties=properties,
-            )
+            ),
+            store=False,
         )
+        (
+            new_status,
+            is_updated,
+        ) = status.storage().create_referring_with_datetime(
+            status,
+            partial(
+                update_last, task_stages=task_stages, properties=properties
+            ),
+        )
+        return t.cast(st.Status, new_status), is_updated
     else:
         return last.update(task_stages=task_stages, properties=properties)
 
@@ -416,7 +401,7 @@ def properties(
     dataspec: st.DataSpec,
     properties: Optional[Mapping[str, str]],
     manager: Optional[Manager] = None,
-) -> st.Status:
+) -> t.Tuple[st.Status, bool]:
     return status(
         dataspec,
         task_stages=None,
@@ -430,7 +415,7 @@ def pending(
     task: str,
     properties: Optional[Mapping[str, str]] = None,
     manager: Optional[Manager] = None,
-) -> st.Status:
+) -> t.Tuple[st.Status, bool]:
     return status(
         dataspec,
         task_stages={task: pending_stage(properties=properties)},
@@ -444,7 +429,7 @@ def processing(
     task: str,
     properties: Optional[Mapping[str, str]] = None,
     manager: Optional[Manager] = None,
-) -> st.Status:
+) -> t.Tuple[st.Status, bool]:
     return status(
         dataspec,
         task_stages={task: processing_stage(properties=properties)},
@@ -458,7 +443,7 @@ def ready(
     task: str,
     properties: Optional[Mapping[str, str]] = None,
     manager: Optional[Manager] = None,
-) -> st.Status:
+) -> t.Tuple[st.Status, bool]:
     return status(
         dataspec,
         task_stages={task: ready_stage(properties=properties)},
@@ -472,7 +457,7 @@ def error(
     task: str,
     properties: Optional[Mapping[str, str]] = None,
     manager: Optional[Manager] = None,
-) -> st.Status:
+) -> t.Tuple[st.Status, bool]:
     return status(
         dataspec,
         task_stages={task: error_stage(properties=properties)},
@@ -488,3 +473,41 @@ def clear_task(dataspec: st.DataSpec, task: str) -> None:
     statuses = last_statuses(dataspec)
     for manager_status in statuses:
         manager_status.clear_task(task)
+
+
+def update_last(
+    status: st.Referring[ProtobufWithUUIDAndDatetime],
+    task_stages: Optional[Mapping[str, st.Stage]],
+    properties: Optional[Mapping[str, str]],
+) -> t.Tuple[st.Referring[ProtobufWithUUIDAndDatetime], bool]:
+    """Returns whether the update should be done, ie transitions
+    are respected and the status that can be updated in case"""
+    # We copy the content of the current status
+    proto = cast(Status, status).protobuf()
+    # We update its timestamp and properties
+    proto.datetime = datetime.now().isoformat()
+    if properties is not None:
+        proto.properties.update(properties)
+        # Protobuf implementation of the merge
+        # does not work as expected on maps...
+    if task_stages is not None:
+        for task in task_stages:
+            stage = task_stages[task].protobuf()
+            if (
+                task in proto.task_stages
+                and proto.task_stages[task].WhichOneof('stage')
+                == task_stages[task].protobuf().WhichOneof('stage')
+                and not stage.properties == proto.task_stages[task].properties
+            ):
+                proto.task_stages[task].properties.update(stage.properties)
+            else:
+                if transition_allowed(
+                    existing_stage=proto.task_stages.get(task, None),
+                    new_stage=stage,
+                    task_name=task,
+                ):
+                    proto.task_stages[task].CopyFrom(stage)
+                else:
+                    return status, False
+    # And return a new UNSTORED data spec object
+    return Status(proto, store=False), True
