@@ -95,7 +95,7 @@ class Storage(Protocol):
         """
         ...
 
-    def update_last_referring(
+    def update_referring_with_datetime(
         self,
         referred: Union[
             Referrable[ProtobufWithUUID],
@@ -106,7 +106,7 @@ class Storage(Protocol):
             [Referring[ProtobufWithUUIDAndDatetime]],
             Tuple[Referring[ProtobufWithUUIDAndDatetime], bool],
         ],
-    ) -> Referring[ProtobufWithUUIDAndDatetime]:
+    ) -> Tuple[Referring[ProtobufWithUUIDAndDatetime], bool]:
         """Update the last referring value of a type atomically
 
         Update the object ``self.last_referring(referred, type_name)``
@@ -138,6 +138,16 @@ class Storage(Protocol):
         """
         ...
 
+    def create_referring_with_datetime(
+        self,
+        value: Referring[ProtobufWithUUIDAndDatetime],
+        update: Callable[
+            [Referring[ProtobufWithUUIDAndDatetime]],
+            Tuple[Referring[ProtobufWithUUIDAndDatetime], bool],
+        ],
+    ) -> Tuple[Referring[ProtobufWithUUIDAndDatetime], bool]:
+        ...
+
     def type_name(
         self, type_name: str
     ) -> Collection[Referrable[ProtobufWithUUID]]:
