@@ -631,7 +631,11 @@ def automatic_user_settings(max_categories: int = 200) -> Transform:
 
 
 def synthetic() -> Transform:
-    """Synthetic transform"""
+    """Synthetic transform. This transform should be
+    called on a dataset with the additional following kwargs:
+    -sampling_ratios: a scalar created by the transform sampling ratios
+    -synthetic_model: a scalar of type synthetic_model
+    """
     return Transform(
         sp.Transform(
             name="Synthetic data",
