@@ -7,6 +7,7 @@ from enum import Enum
 import datetime as dt
 import logging
 import typing as t
+import warnings
 
 import numpy as np
 import pandas as pd
@@ -37,7 +38,7 @@ try:
     PrivateQuery = RealPrivateQuery
 except ImportError:
     PrivateQuery = t.Any  # type: ignore
-    logger.warning(
+    warnings.warn(
         "`sarus_differential_privacy` not installed. "
         "DP primitives not available."
     )
@@ -48,7 +49,7 @@ try:
     Task = RealTask
 except ImportError:
     Task = t.Any  # type: ignore
-    logger.warning(
+    warnings.warn(
         "`sarus_query_builder` not installed. DP methods not available."
     )
 
@@ -297,6 +298,9 @@ class Dataset(DataSpec[sp.Dataset], t.Protocol):
     def is_synthetic(self) -> bool:
         ...
 
+    def is_dp(self) -> bool:
+        ...
+
     def is_protected(self) -> bool:
         ...
 
@@ -386,6 +390,9 @@ class Scalar(DataSpec[sp.Scalar], t.Protocol):
     def is_random_seed(self) -> bool:
         """Is the scalar a random seed."""
 
+    def is_synthetic_model(self) -> bool:
+        """is the scalar a synthetic model"""
+
     def value(self) -> DataSpecValue:
         ...
 
@@ -488,6 +495,12 @@ class Schema(Referring[sp.Schema], t.Protocol):
     def data_type(self) -> Type:
         ...
 
+    def private_tables(self) -> t.List[Path]:
+        ...
+
+    def public_tables(self) -> t.List[Path]:
+        ...
+
 
 class Status(Referring[sp.Status], t.Protocol):
     """A python abstract class to describe status"""
@@ -506,7 +519,7 @@ class Status(Referring[sp.Status], t.Protocol):
         self,
         task_stages: t.Optional[t.Mapping[str, Stage]],
         properties: t.Optional[t.Mapping[str, str]],
-    ) -> Status:
+    ) -> t.Tuple[Status, bool]:
         ...
 
     def task(self, task: str) -> t.Optional[Stage]:
@@ -531,7 +544,7 @@ class Status(Referring[sp.Status], t.Protocol):
     ):  # TODO: Maybe find a better name, but this was shadowing the actual manager of this object.  # noqa: E501
         ...
 
-    def clear_task(self, task: str) -> t.Optional[Status]:
+    def clear_task(self, task: str) -> t.Tuple[Status, bool]:
         """Creates a new status removing the task specified.
         If the task does not exist, nothing is created"""
 
@@ -832,6 +845,7 @@ class ConstraintKind(Enum):
     PEP = sp.ConstraintKind.PEP
     DP = sp.ConstraintKind.DP
     PUBLIC = sp.ConstraintKind.PUBLIC
+    MOCK = sp.ConstraintKind.MOCK
 
 
 class SQLDialect(Enum):
